@@ -15,20 +15,42 @@
 #include <openssl/evp.h>
 #include <openssl/ssl.h>
 #include <pthread.h>
+#include <getopt.h>
 #include <microhttpd.h>
 #include <jansson.h>
 
-#define SERVER "irc.libera.chat"
-#define PORT 6697
-#define NICK "lunabot"
-#define CHANNEL "#lunar"
-#define WEBHOOK_PORT 3000
+const char *lunabot_version_string = "0.2.1";
 
-const char *lunabot_version = "0.1.6";
+static const struct option long_options[] = {
+	{"help", no_argument, NULL, 'h'},
+	{"version", no_argument, NULL, 'V'},
+	{"channel", required_argument, NULL, 'c'},
+	{"debug", no_argument, NULL, 'd'},
+	{"irc-port", required_argument, NULL, 'p'},
+	{"irc-server", required_argument, NULL, 's'},
+	{"log", required_argument, NULL, 'l'},
+	{"nick", required_argument, NULL, 'n'},
+	{"webhook-port", required_argument, NULL, 'w'},
+	{NULL, 0, NULL, 0}
+};
+static const char *short_options = "hVc:dl:n:p:s:w:";
 
+#define DEFAULT_IRC_SERVER   "irc.libera.chat"
+#define DEFAULT_IRC_PORT     6697
+#define DEFAULT_NICK         "lunabot"
+#define DEFAULT_CHANNEL      "#lunar-lunabot"
+#define DEFAULT_WEBHOOK_PORT 3000
+#define DEFAULT_LOG_FILENAME "lunabot.log"
+
+unsigned int debug;
 unsigned int mainloopend;
 int irc_sock;
-char server_ip[16];
+char *irc_server_hostname;
+char irc_server_ip[16];
+unsigned int irc_server_port;
+unsigned int webhook_port;
+char *nick;
+char *channel;
 SSL *pSSL;
 #define BUFFER_SIZE 1024
 char buffer[BUFFER_SIZE];
@@ -59,16 +81,17 @@ unsigned int ignore_pending = 1;
 #define LOCAL 0
 #define IN    1
 #define OUT   2
-char *log_filename = "lunabot.log";
+char *log_filename;
+unsigned int disable_logging;
+
+void LunabotHelp(void) {
+printf("lunabot option usage: lunabot --help/-h | --version/-V | --debug/-d |\n"
+	"\t--channel/-c NAME | --nick/-n NAME | --irc-port/-p NUMBER |\n"
+	"\t--irc-server/-s HOSTNAME | --log/-l FILENAME, off | \n"
+	"\t--webhook-port/-w NUMBER\n");
+}
 
 void Log(unsigned int direction, char *text) {
-	FILE *log_fp = fopen(log_filename, "a+");
-	if (log_fp == NULL) {
-		fprintf(stderr, "lunabot::Log() error: Cannot open '%s': %s\n",
-			log_filename, strerror(errno));
-		exit(1);
-	}
-
 	char *dirstr;
 	if (direction == LOCAL)
 		dirstr = "||";
@@ -84,11 +107,6 @@ void Log(unsigned int direction, char *text) {
 	struct timeval tv0;
 	gettimeofday(&tv0, NULL);
 
-	fprintf(log_fp, "%04d%02d%02d-%02d:%02d:%02d.%06ld %s##%s##\n",
-		tm0->tm_year+1900, tm0->tm_mon+1, tm0->tm_mday,
-		tm0->tm_hour, tm0->tm_min, tm0->tm_sec, tv0.tv_usec,
-		dirstr, text);
-
 	// Show message in console with colors
 	fprintf(stdout, "\033[00;36m%04d%02d%02d-%02d:%02d:%02d.%06ld %s"
 		"##\033[00m%s\033[00;36m##\033[00m\n", 
@@ -96,6 +114,21 @@ void Log(unsigned int direction, char *text) {
 		tm0->tm_hour, tm0->tm_min, tm0->tm_sec, tv0.tv_usec,
 		dirstr, text);
 	
+	if (disable_logging)
+		return;
+
+	FILE *log_fp = fopen(log_filename, "a+");
+	if (log_fp == NULL) {
+		fprintf(stderr, "lunabot::Log() error: Cannot open '%s': %s\n",
+			log_filename, strerror(errno));
+		exit(1);
+	}
+	
+	fprintf(log_fp, "%04d%02d%02d-%02d:%02d:%02d.%06ld %s##%s##\n",
+		tm0->tm_year+1900, tm0->tm_mon+1, tm0->tm_mday,
+		tm0->tm_hour, tm0->tm_min, tm0->tm_sec, tv0.tv_usec,
+		dirstr, text);
+
 	fclose(log_fp);
 }
 
@@ -109,7 +142,8 @@ char *GetIP(char *hostname) {
 	hints.ai_socktype = SOCK_STREAM;
 
 	if ((status = getaddrinfo(hostname, NULL, &hints, &res)) != 0) {
-		sprintf(buffer_log, "lunabot::GetIP() error: getaddrinfo() failed: %s", gai_strerror(status));
+		sprintf(buffer_log, "lunabot::GetIP() error: getaddrinfo() failed: %s",
+			gai_strerror(status));
 		Log(LOCAL, buffer_log);
 		return NULL;
 	}
@@ -119,9 +153,9 @@ char *GetIP(char *hostname) {
 		if (p->ai_family == AF_INET) { // IPv4
 			struct sockaddr_in *ipv4 = (struct sockaddr_in *)p->ai_addr;
 			addr = &(ipv4->sin_addr);
-			inet_ntop(p->ai_family, addr, server_ip, sizeof(server_ip));
+			inet_ntop(p->ai_family, addr, irc_server_ip, sizeof(irc_server_ip));
 			freeaddrinfo(res); // Cleanup
-			return server_ip;
+			return irc_server_ip;
 		}
 		else
 			continue;
@@ -135,8 +169,9 @@ char *GetIP(char *hostname) {
 // Function to send messages to the IRC channel
 void SendIrcMessage(const char *message) {
 	Log(OUT, (char *)message);
-	char buffer_msg[4096];
-	snprintf(buffer_msg, sizeof(buffer_msg), "PRIVMSG %s :%s\r\n", CHANNEL, message);
+	char buffer_msg[BUFFER_SIZE * 16];
+	snprintf(buffer_msg, sizeof(buffer_msg), "PRIVMSG %s :%s\r\n",
+		channel, message);
 	SSL_write(pSSL, buffer_msg, strlen(buffer_msg));
 }
 
@@ -147,26 +182,29 @@ void *IrcConnect(void *arg) {
 	// Create socket
 	irc_sock = socket(AF_INET, SOCK_STREAM, 0);
 	if (irc_sock < 0) {
-		sprintf(buffer_log, "lunabot::IrcConnect() error: socket() failed: %s", strerror(errno));
+		sprintf(buffer_log, "lunabot::IrcConnect() error: socket() failed: %s",
+			strerror(errno));
 		Log(LOCAL, buffer_log);
 		exit(1);
 	}
 
-	char *ret = GetIP(SERVER);
+	char *ret = GetIP(irc_server_hostname);
 	if (ret == NULL) {
-		sprintf(buffer_log, "lunabot::IrcConnect() error: Cannot get an IP for '%s'", SERVER);
+		sprintf(buffer_log, "lunabot::IrcConnect() error: Cannot get an IP for '%s'",
+			irc_server_hostname);
 		Log(LOCAL, buffer_log);
 		close(irc_sock);
 		exit(1);
 	}
 
 	server_addr.sin_family = AF_INET;
-	server_addr.sin_port = htons(PORT);
-	server_addr.sin_addr.s_addr = inet_addr(server_ip);
+	server_addr.sin_port = htons(irc_server_port);
+	server_addr.sin_addr.s_addr = inet_addr(irc_server_ip);
 
 	// Connect to IRC server
 	if (connect(irc_sock, (struct sockaddr *)&server_addr, sizeof(server_addr)) < 0) {
-		sprintf(buffer_log, "lunabot::IrcConnect() error: connect() failed: %s", strerror(errno));
+		sprintf(buffer_log, "lunabot::IrcConnect() error: connect() failed: %s",
+			strerror(errno));
 		Log(LOCAL, buffer_log);
 		close(irc_sock);
 		exit(1);
@@ -192,14 +230,14 @@ void *IrcConnect(void *arg) {
 
 	BIO *bio = BIO_new_socket(irc_sock, BIO_CLOSE);
 	SSL_set_bio(pSSL, bio, bio);
-	SSL_set1_host(pSSL, SERVER);
+	SSL_set1_host(pSSL, irc_server_hostname);
 	SSL_connect(pSSL);
 
 	// Send basic IRC commands
-	sprintf(buffer, "NICK %s\r\n", NICK);
+	sprintf(buffer, "NICK %s\r\n", nick);
 	SSL_write(pSSL, buffer, strlen(buffer));
 
-	sprintf(buffer, "USER %s 0 * :GitHub PR IRC bot\r\n", NICK);
+	sprintf(buffer, "USER %s 0 * :IRC bot for Github webhooks\r\n", nick);
 	SSL_write(pSSL, buffer, strlen(buffer));
 
 	const char *env_pass = getenv("LUNABOT_NICKSERV_PASSWORD");
@@ -215,20 +253,19 @@ void *IrcConnect(void *arg) {
 			Log(LOCAL, buffer_log);
 			exit(1);
 		}
-		else {
-			char pass[BUFFER_SIZE - 30];
-			fgets(pass, BUFFER_SIZE - 31, fp);
-			fclose(fp);
-			if (pass[strlen(pass)-1] == '\n')
-				pass[strlen(pass)-1] = '\0';
-			sprintf(buffer, "PRIVMSG NickServ :IDENTIFY %s\r\n", pass);
-			Log(OUT, "PRIVMSG NickServ :IDENTIFY ********");
-			SSL_write(pSSL, buffer, strlen(buffer));
-		}
+
+		char pass[BUFFER_SIZE - 30];
+		fgets(pass, BUFFER_SIZE - 31, fp);
+		fclose(fp);
+		if (pass[strlen(pass)-1] == '\n')
+			pass[strlen(pass)-1] = '\0';
+		sprintf(buffer, "PRIVMSG NickServ :IDENTIFY %s\r\n", pass);
+		Log(OUT, "PRIVMSG NickServ :IDENTIFY ********");
+		SSL_write(pSSL, buffer, strlen(buffer));
 	}
 // Not logged in yet, exposes hostmask, needs to be sent manually in the terminal
-//	sprintf(buffer, "JOIN %s\r\n", CHANNEL);
-//	SSL_write(pSSL, buffer, strlen(buffer));
+	sprintf(buffer, "JOIN %s\r\n", channel);
+	SSL_write(pSSL, buffer, strlen(buffer));
 
 	// Listen for server messages
 	while (1) {
@@ -239,9 +276,9 @@ void *IrcConnect(void *arg) {
 			break;
 
 		if (buffer[bytes-1] == '\n')
-			buffer[bytes-1] = '\0';   // Remove '\n'
+			buffer[bytes-1] = '\0'; // Remove ending '\n'
 		if (buffer[bytes-2] == '\r')
-			buffer[bytes-2] = '\0'; // Remove '\r'
+			buffer[bytes-2] = '\0'; // Remove ending '\r'
 		
 		
 		// Respond to ping requests with a pong message
@@ -258,24 +295,44 @@ void *IrcConnect(void *arg) {
 	return NULL;
 }
 
+void IrcConnectStart(void) {
+	pthread_t irc_thread;
+	pthread_attr_t attr;
+	pthread_attr_init(&attr);
+	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+	pthread_create(&irc_thread, &attr, IrcConnect, NULL);
+	pthread_detach(irc_thread);
+	pthread_attr_destroy(&attr);
+}
+
 // Function to verify the GitHub webhook signature
 int VerifySignature(const char *payload, const char *signature) {
+	if (debug)
+		fprintf(stderr, "VerifySignature() started\n");
+
 	unsigned int hash_len = 32;
 	unsigned char hash[hash_len];
 	char *secret = (char *)getenv("LUNABOT_WEBHOOK_SECRET");
 
-	unsigned int len;
+	unsigned int secret_len = 0;
 	if (secret != NULL)
-		len = strlen(secret);
+		secret_len = strlen(secret);
+
+	if (secret == NULL || secret_len == 0) {
+		secret = malloc(BUFFER_SIZE);
+		if (secret == NULL) {
+			Log(LOCAL, "lunabot::VerifySignature(): Cannot allocate memory");
+			exit(1);
+		}
+		memset(secret, 0, BUFFER_SIZE);
 
-	if (secret == NULL || len == 0) {
 		FILE *fp = fopen(".secret", "r");
 		if (fp == NULL) {
 			Log(LOCAL, "lunabot::VerifySignature(): .secret file not found!");
 			exit(1);
 		}
 		else {
-			fgets(secret, 1023, fp);
+			fgets(secret, BUFFER_SIZE - 1, fp);
 			fclose(fp);
 		
 			// Strip newline ending
@@ -309,6 +366,9 @@ int VerifySignature(const char *payload, const char *signature) {
 }
 
 void ParseJsonData(char *json_data) {
+	if (debug)
+		fprintf(stderr, "ParseJsonData() started\n");
+
 	json_t *root;
 	json_error_t error;
 	root = json_loads(json_data, 0, &error);
@@ -401,7 +461,8 @@ void ParseJsonData(char *json_data) {
 				if (json_is_object(repo)) {
 					json_t *repo_name = json_object_get(repo, "name");
 					if (json_is_string(repo_name)) {
-						if (strcmp(json_string_value(repo_name), "moonbase-core") != 0) {
+						if (strcmp(json_string_value(repo_name),
+						  "moonbase-core") != 0) {
 							json_decref(root);
 							return;
 						}
@@ -418,7 +479,7 @@ void ParseJsonData(char *json_data) {
 				json_t *label_name = json_object_get(label, "name");
 				char message[512];
 				snprintf(message, sizeof(message), 
-						 "[%sLabels%s]: %s added the %s label to '%s' - %s",
+						 "[%sLabels%s]: %s added the '%s' label to '%s' - %s",
 						 LIGHT_GREEN, NORMAL,
 						 json_string_value(username),
 						 json_string_value(label_name),
@@ -438,7 +499,8 @@ void ParseJsonData(char *json_data) {
 				if (json_is_object(repo)) {
 					json_t *repo_name = json_object_get(repo, "name");
 					if (json_is_string(repo_name)) {
-						if (strcmp(json_string_value(repo_name), "moonbase-core") != 0) {
+						if (strcmp(json_string_value(repo_name),
+						  "moonbase-core") != 0) {
 							json_decref(root);
 							return;
 						}
@@ -455,7 +517,7 @@ void ParseJsonData(char *json_data) {
 				json_t *label_name = json_object_get(label, "name");
 				char message[512];
 				snprintf(message, sizeof(message), 
-						 "[%sLabels%s]: %s removed the %s label to '%s' - %s",
+						 "[%sLabels%s]: %s removed the '%s' label to '%s' - %s",
 						 LIGHT_GREEN, NORMAL,
 						 json_string_value(username),
 						 json_string_value(label_name),
@@ -535,7 +597,7 @@ static enum MHD_Result WebhookCallback(void *cls, struct MHD_Connection *connect
 				data, MHD_RESPMEM_PERSISTENT);
 		int ret = MHD_queue_response(connection, 401, response401);
 		MHD_destroy_response(response401);
-		Log(LOCAL, "Webhook signature verification failed!");
+		Log(LOCAL, "Webhook signature missing from the HTTP header!");
 		return ret;
 	}
 
@@ -617,29 +679,104 @@ static enum MHD_Result WebhookCallback(void *cls, struct MHD_Connection *connect
 
 // Webhook server thread
 void WebhookServerStart(void) {
-	httpdaemon = MHD_start_daemon(MHD_USE_SELECT_INTERNALLY, WEBHOOK_PORT, NULL, NULL,
+	httpdaemon = MHD_start_daemon(MHD_USE_SELECT_INTERNALLY, webhook_port, NULL, NULL,
 							  &WebhookCallback, NULL, MHD_OPTION_END);
 	if (!httpdaemon) {
 		Log(LOCAL, "lunabot::WebhookServerStart(): Failed to start HTTP server");
 		exit(1);
 	}
 	else {
-		sprintf(buffer_log, "Webhook server running on port %d", WEBHOOK_PORT);
+		sprintf(buffer_log, "Webhook server running on port %d", webhook_port);
 		Log(LOCAL, buffer_log);
 	}
 }
 
+void ParseArgs(int *argc, char **argv) {
+	int c;
+	while (1) {
+		c = getopt_long(*argc, argv, short_options, long_options, NULL);
+		if (c == -1)
+			break;
+
+		switch (c) {
+		case 'h': // --help
+			LunabotHelp();
+			exit(0);
+			break;
+		case 'V': // --version
+			printf("lunabot %s\n", lunabot_version_string);
+			exit(0);
+			break;
+		case 'c': // --channel
+			if (optarg != NULL && strlen(optarg))
+				channel = strdup(optarg);
+
+			break;
+		case 'd':
+			debug = 1;
+			break;
+		case 'l': // --log
+			if (optarg != NULL && strlen(optarg)) {
+				if (strcmp(optarg, "off") == 0)
+					disable_logging = 1;
+				else
+					log_filename = strdup(optarg);
+			}
+
+			break;
+		case 'n': // --nick
+			if (optarg != NULL && strlen(optarg))
+				nick = strdup(optarg);
+
+			break;
+		case 'p': // --irc-port
+			if (optarg != NULL && strlen(optarg))
+				irc_server_port = (unsigned int)atoi(optarg);
+
+			break;
+		case 's': // --irc-server
+			if (optarg != NULL && strlen(optarg))
+				irc_server_hostname = strdup(optarg);
+
+			break;
+		case 'w': // --webhook-port
+			if (optarg != NULL && strlen(optarg))
+				webhook_port = (unsigned int)atoi(optarg);
+
+			break;
+		default:
+			fprintf(stderr, "lunabot::ParseArgs() warning: Unknown "
+				"option: %d (%c)\n", c, (char)c);
+			break;
+		}
+	}
+}
+
 // Program entry point
-int main() {
+int main(int argc, char **argv) {
+	ParseArgs(&argc, argv);
+
+	if (!irc_server_hostname)
+		irc_server_hostname = strdup(DEFAULT_IRC_SERVER);
+
+	if (!irc_server_port)
+		irc_server_port = DEFAULT_IRC_PORT;
+		
+	if (!webhook_port)
+		webhook_port = DEFAULT_WEBHOOK_PORT;
+
+	if (!nick)
+		nick = strdup(DEFAULT_NICK);
+
+	if (!channel)
+		channel = strdup(DEFAULT_CHANNEL);
+
+	if (!log_filename)
+		log_filename = strdup(DEFAULT_LOG_FILENAME);
+
 	WebhookServerStart();
 
-	pthread_t irc_thread;
-	pthread_attr_t attr;
-	pthread_attr_init(&attr);
-	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-	pthread_create(&irc_thread, &attr, IrcConnect, NULL);
-	pthread_detach(irc_thread);
-	pthread_attr_destroy(&attr);
+	IrcConnectStart();
 
 	// Start reading user input from the terminal and process per-line
 	char buffer_line[BUFFER_SIZE];
